% A puzzle feladat megoldsa - listkkal.
% A feladat paramtere egy lista, melynek
% mrete NxN.

% puzzle(Kezdet,Ut) - a kezdeti llapotbl visszatrt
% egy utat, mellyel a Cl-llapotba jutunk.
puzzle(Kezdet,Ut) :-
    length(Kezdet,Nk),
    N is ceiling(sqrt(Nk)),
    N * N =:= Nk,
    generate(Nk,Cel),
    ut(N,Kezdet,Cel,Ut).

% generate(N,Lista) - generlja az [1,N-1,0]
% listt, ahol az utols elem nulla.
generate(N,Veg):-
    N1 is N - 1,
    generate(N1,[0],Veg).

% listageneralo egy AKK-ral.
generate(0,Veg,Veg):-!.
generate(K,L,Veg) :-
    K1 is K - 1,
    generate(K1,[K|L],Veg).

% ut(N,Kezdet,Cel,Ut) - az NxN-es puzzle-ben keres
% utat a Kezdet s a Cl kztt. Az Ut-ban a NULLS
% elem mozgsa van.
ut(_,Kezd,Kezd,[]) :- !.
ut(N,Kezd,Cel,Utak) :-
    ut(N,Kezd,[],[Kezd],Cel,Utak).

% ut(N,Kezd,Tt,llList,Cl,t) - a Kezd
% s Cl-llapotok kztt tall egy utat.
% A Tt-ba gyujtjk a lpseket.
%$ Megllunk, ha Kezd s Cl megegyezik.
ut(_,Cel,TUt,_,Cel,Ut):-
    reverse(TUt,Ut), !.


% Lpnk a Kezd-bol egyet s keressk az
% utat a Cl-ba. Az llList trolja a
% mr bejrt llapotokat.
ut(N,Kezd,TUt,Volt,Cel,VUt) :-
    egylepes(N,Kezd,IEgy,Koz),
    \+ member(Koz,Volt),
    length(Volt,Nb),Nb < N*N,
    ut(N,Koz,[IEgy|TUt],[Kezd|Volt],Cel,VUt).

% egylepes(Honnan,Merre,Hova) - a Honnan-bl a Hov-ba jutunk az
% llapottrben. Merre jelzi, hogy az RES pozci merre mozdul.
egylepes(N,Honnan,Irany,Hova) :-
    zerusPoz(Honnan,IndN),
    ujInd(N,IndN,IndLKi,Irany),
    cserel(Honnan,IndN,IndLKi,Hova).

% ujInd(N,IndReg,IndUj,Irany) - az Irnyba
% lps ltal generlt j index.
ujInd(N,IndReg,IndUj,Irany) :-
    ijPoz(N,IndReg,I,J),
    member(Pi/Pj/Irany, %
      [ 1/0/le,0/(-1)/bal,
      (-1)/0/fel,0/1/jobb] ),
    Iuj is I + Pi, Iuj > 0, Iuj =< N,
    Juj is J + Pj, Juj > 0, Juj =< N,
    IndUj is N*(Iuj-1) + Juj.

% ijPoz(N,K,I,J) - az NxN-es tblban a
% K-hoz tartoz indexeket trti vissza
ijPoz(N,K,I,J) :-
    J is ((K-1) rem N) + 1,
    I is ((K-1)//N) +1.

% cserel(Lista,Ind1,Ind2,Valasz) -
% cserli az Ind1 s Ind2 elemeket.
cserel(LBe,Ind1,Ind2,LKi) :-
    Ind1<Ind2,
    csere_ord(LBe,Ind1,Ind2,LKi),  !.
cserel(LBe,Ind1,Ind2,LKi) :-
    csere_ord(LBe,Ind2,Ind1,LKi).
% csere_ord(Lista,I1,I2,Valasz), !I1<I2!
csere_ord(LBe,Ind1,Ind2,LKi) :-
    allista(LBe,Ind1,L_E1,E1,L_U),
    I22 is Ind2-Ind1,
    allista(L_U,I22,L_EU1,E2,L_UU),
    flatten([L_E1,E2,L_EU1,E1,L_UU],LKi).

allista([Elem|Marad],1,[],Elem,Marad) :- !.
allista([Fej|Marad],K,[Fej|EMarad],Elem,Uto) :-
    K>1,
    K1 is K - 1,
    allista(Marad,K1,EMarad,Elem,Uto).

% zerusPoz(Lista,Ind) - a Listaban
% megkeresi a nullt.
zerusPoz([0|_],1):- !.
zerusPoz([_|Marad],Ind) :-
    zerusPoz(Marad,IM),
    Ind is IM + 1.

%%%%%%%%%%%%%%%%% MEGJELENITES %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% tikzSor(Kezdet,Ut) - egy tikz sorozatba kirja a
% puzzle megoldsokat. Ha a msodik argumentum RES,
% csak egy llapotot r ki.
tikzSor(Kezdet,Ut) :-
    length(Kezdet,N2), N is ceiling(sqrt(N2)),
    writeln('%---------------------\n%---Generlt szveg---'),
    tikzListaKiir(N,Kezdet,Ut),
    writeln('%-----Szveg vge-----\n%---------------------').

%% tikzListaKiir(N,Kezdet,Ut) - a Kezdet-bol indulva kirja
% a puzzle lpseket. RES T-nl egy TBLT runk ki.
tikzListaKiir(N,Kezdet,[]) :-
    tikzElemKiir(N,Kezdet).
tikzListaKiir(N,Kezdet,[Irany|MarUt]) :-
    tikzElemKiir(N,Kezdet),
    egylepes(N,Kezdet,Irany,Kov),
    iranyKiir(Irany),
    tikzListaKiir(N,Kov,MarUt).

iranyKiir(Irany) :-
    member(Irany/Dir,[bal/'Right',jobb/'Left',le/'Up',fel/'Down']),
    write('\%\n$\\stackrel{\\pmb \\'),write(Dir),
    writeln('arrow}{\\pmb \\longrightarrow}$\n\%').

% tikzElemKiir(N,Puzzle) - egy TIKZ brba kir egy llapotot
tikzElemKiir(N,Puzzle) :-
    writePre(N),
    tikzPuzzleLista(N,1,Puzzle),
    writePost.

% tikzPuzzleLista(N,I,J,Puzzle) - kirja az (I,J) pozciba
% a Lista fejt, nveli az rtkeket.
tikzPuzzleLista(_,_,[]) :- !.
tikzPuzzleLista(N,K,[0|MarPuz]) :-
    K1 is K + 1,
    tikzPuzzleLista(N,K1,MarPuz),
    !.
tikzPuzzleLista(N,K,[Fej|MarPuz]) :-
    ijPoz(N,K,I,J),
    I1 is N + 1 - I,
    write('    \\node[draw] at ('), write(J),write(','),write(I1),
    write(') {\\bfseries\\tt '),write(Fej),writeln('};'),
    K1 is K + 1,
    tikzPuzzleLista(N,K1,MarPuz).

% Preambulum a TIKZ kphez
writePre(N) :-
    writeln('\\begin{tikzpicture}[line width=1.2pt,scale=0.5]'),
    write('  \\draw[step=1cm,gray!25!red!25!] (-0.1,-0.1) grid ('),
    write(N),write('.1,'),write(N),writeln('.1);'),
    writeln('  \\begin{scope}[color=blue!35!green!,minimum size=0.2cm,\%'),
    writeln('      rectangle,xshift=-0.5cm,yshift=-0.5cm,inner sep=3pt,\%'),
    writeln('      outer sep=4pt]').


writePost :-  % vge - nincs paramter
    writeln('  \\end{scope}\n\\end{tikzpicture}').
